#############
# $Id: CheckCifDif.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-09-21
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/analysis/CheckCifDif.pm $

package npg_pipeline::analysis::CheckCifDif;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };
use File::Spec::Functions;

extends qw{npg_pipeline::base};
with qw{npg_common::roles::run::path_info};
with qw{npg_common::roles::run::intensities::config};
with qw{npg_common::roles::run::long_info};

Readonly::Scalar our $CIF_HEADER_SIZE => 13;
Readonly::Scalar our $CIF_MAGIC       => q(CIF);
Readonly::Scalar our $FOUR            => 4;

=head1 NAME

npg_pipeline::analysis::CheckCifDif

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  use npg_pipeline::analysis::CheckCifDif;
  my $oCheckCifDif = npg_pipeline::analysis::CheckCifDif->new();

=head1 DESCRIPTION

This module checks for and fixes any missing or broken .cif and .dif files that are expected for this run.
Ideally this should be run early in the pipeline, so that any processes that expect to use these are not run
until they are fixed.

=head1 SUBROUTINES/METHODS

=head2 check_all_cif_and_dif

run the check (and fixing) for all cif and dif files

returns an empty array

=cut

sub check_all_cif_and_dif {
  my ( $self, $arg_refs ) = @_;

  $self->_check_cif_files();
  $self->_check_dif_files();

  return ();
}

=head2 check_and_fix_cif_files

run the the check and fixing for all cif files only

returns an empty array

=cut

sub check_and_fix_cif_files {
  my ( $self, $arg_refs ) = @_;

  $self->_check_cif_files();

  return ();
}

##############
# private methods

# check cif files (and fix if no told to croak)
sub _check_cif_files {
  my ( $self ) = @_;

  my $intensity_path = $self->intensity_path();
  $self->log( $intensity_path );
  my $lane_tile_clustercount = $self->lane_tile_clustercount();

  my $err_count = 0;
  for my $cycle ( $self->first_cycle() .. $self->last_cycle() ){

    while ( my ( $lane , $tile_clustercount_href ) = each %{ $lane_tile_clustercount } ) {

      for my $tile ( keys %{ $tile_clustercount_href } ) {
        my $f = catfile( $intensity_path, sprintf( 'L%03d', $lane ), qq{C$cycle.1}, sprintf q{s_%s_%d.cif}, $lane, $tile );

        eval {

          open my $fh, q(<), $f or croak qq{cannot open file $f\n};
          my $cif_header;
          read( $fh, $cif_header, $CIF_HEADER_SIZE ) == $CIF_HEADER_SIZE or croak qq{file shorter than header\n};
          my ( $magic, $version, $entry_size, $first_cycle, $num_cycles, $num_clusters ) = unpack ( q(A3 CC vvV), $cif_header ); ## no critic (CodeLayout::ProhibitParensWithBuiltins)

          if ( $magic ne $CIF_MAGIC ) {
            croak qq{no CIF magic\n};
          }

          if ( not defined $tile_clustercount_href->{ $tile } ) {

            $tile_clustercount_href->{ $tile } = $num_clusters;

          } elsif ( $tile_clustercount_href->{ $tile } != $num_clusters ) {

            croak qq{inconsistent cluster count\n};

          }

          if ( $first_cycle != $cycle ) {
            croak qq{cycle metadata disagrees with location\n};
          }

          my $rs = $FOUR * $entry_size * $num_cycles * $num_clusters;
          my $dummy;
          read ( $fh, $dummy, $rs ) == $rs or croak qq{file data too short\n};
          read ( $fh, $dummy, 1   ) == 0   or croak qq{file data too long\n};
	        close $fh or croak qq{could not close $f};
          $rs += $CIF_HEADER_SIZE;
	        croak qq{file size not right for expect cluster size\n} if ( $rs != -s $f );
          1;
        } or do {
          my $msg = qq{Lane $lane, Tile $tile, Cycle $cycle};

          if ( $self->verbose() ) {
            $self->log( qq{Error: $EVAL_ERROR} );
          }

          $err_count++;

          if ( $self->fix_broken_files() ) {
	          my $num_clusters = $tile_clustercount_href->{$tile};
	          croak qq{cycle $cycle, lane $lane, tile $tile, unknown number of clusters\n} if not defined $num_clusters;
            $self->_fix_cif_file( {
              cycle => $cycle,
              lane  => $lane,
              tile  => $tile,
              file  => $f,
              num_clusters => $num_clusters,
              msg => $msg,
            } );
            $self->log( qq{$msg, fixed} );
            $err_count--;
          }
          if ( ! $self->fix_broken_files() && $self->verbose() ) {
            $self->log( qq{$msg, not fixed} );
          }
        };
      }
    }
  }

  if ( ! $self->fix_broken_files() && $err_count ) {
    croak qq{$err_count error cif files found};
  }

  return $err_count;
}

# check all dif files exist
sub _check_dif_files {
  my ( $self ) = @_;

  my $lane_tile_clustercount = $self->lane_tile_clustercount();

  my $intensity_path = $self->intensity_path();

  my $err_count = 0;
  for my $cycle ( $self->first_cycle() .. $self->last_cycle() ){

    while ( my ( $lane , $tile_clustercount_href ) = each %{ $lane_tile_clustercount } ) {

      for my $tile ( keys %{ $tile_clustercount_href } ) {
        my $f = catfile( $intensity_path, sprintf( 'L%03d', $lane ), qq{C$cycle.1}, sprintf q{s_%s_%d.dif}, $lane, $tile );

        eval {

           if ( ! -e $f ) {
             croak qq{Unable to find $f};
           }
#          open my $fh, q(<), $f or croak qq{cannot open file $f\n};

#          close $fh;
          1;
        } or do {
          my $msg = qq{cycle $cycle, lane $lane, tile $tile};
          if ( $self->verbose() ) {
            $self->log( qq{Error: $EVAL_ERROR} );
          }
          $err_count++;

          ##########
          # as things stand, we don't fix the dif files anyway
#          if ( ! $self->croak_if_broken_cif_dif() ) {
#            fix_dif_file( $cycle, $lane, $tile, $f );
#            $self->log( qq{$msg, fixed\n} );
#            $err_count--;
#          }
          if ( ! $self->fix_broken_files() && $self->verbose() ) {
            $self->log( qq{$msg, not fixed} );
          }
        };

      }
    }
  }

#  if ( $self->croak_if_broken_cif_dif() && $err_count ) {
  if ( $err_count ) {
    croak qq{$err_count error dif files found};
  }

  return $err_count;
}

# method for fixing cif files for the pipeline to continue
sub _fix_cif_file {
  my ( $self, $arg_refs ) = @_;
  my $cycle = $arg_refs->{cycle};
  my $lane  = $arg_refs->{lane};
  my $tile  = $arg_refs->{tile};
  my $file  = $arg_refs->{file};
  my $num_clusters = $arg_refs->{num_clusters};

  $self->log( qq{Fixing cif file for $arg_refs->{msg}} );

  $file ||= qq{s_$lane} . qq{_$tile} . q{.cif};
  my $fno = $file . q{.ORIGINAL};
  if ( -e $fno ) {
    croak qq{$fno already exists};
  }

  if ( -e $file ) {
    rename $file, $fno or croak qq{could not move $file to $fno};
  }

  open my $fh, q{>}, $file or croak qq{could not open $file};
  syswrite $fh, pack( q(A3 CC vvV), $CIF_MAGIC, 1, 2, $cycle, 1, $num_clusters ); ## no critic (CodeLayout::ProhibitParensWithBuiltins)
  my $i = 0;
  while ( $i < ( $FOUR * $num_clusters ) ) {
    syswrite $fh, pack q(s), 0;
    $i++;
  }
  close $fh or croak qq{could not close $file};

  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item File::Spec::Functions

=item npg_common::roles::run::path_info

=item npg_common::roles::run::intensities::config

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
