#############
# $Id: generation.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-07
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/archive/file/generation.pm $

package npg_pipeline::archive::file::generation;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };
use File::Spec;
use File::Basename;
use Cwd 'abs_path';
use npg_common::sequence::reference::roles::find;
use Moose::Meta::Class;

extends q{npg_pipeline::archive};
with qw{npg_pipeline::roles::business::file_constructs};
with qw{npg_common::roles::run::lane::tag_info};

=head1 NAME

npg_pipeline::archive::file::generation

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  package My::archive::file::generation::sub_class;
  use Moose;
  extends qw{npg_pipeline::archive::file::generation};

  ...
  1;

  my $oMySubClass = My::archive::file::generation::sub_class->new();

=head1 DESCRIPTION

Base class for individual archival file generator object modules, which exposes private methods which
know how to construct the correct filenames for files that may need to be submitted to a command line

=head1 SUBROUTINES/METHODS

=head2 generate_fastq_and_check_name_parts

generates the unique name parts for use in fastq, fastqcheck and gcfreq filenames

  my @file_name_parts = $oMySubClass->generate_fastq_and_check_name_parts($arg_refs);

=cut

sub generate_fastq_and_check_name_parts {
  my ($self,$arg_refs) = @_;

  my @file_names;
  my @positions = ();
  if ( $arg_refs->{array_string} ) {
    push @positions, $self->lsb_jobindex();
  } else {
   @positions = $self->positions( $arg_refs );
  }

  my $pr = $self->is_paired_read();
  my $ir = $self->is_indexed();

  foreach my $position ( @positions )  {
    if ( $pr or $ir ) {
      push @file_names, $position . q{_1};
    } else{
      push @file_names, $position;
    }
    if ( $ir ) {
      push @file_names, $position . q{_t};
    }
    if ( $pr ) {
      push @file_names, $position . q{_2};
    }
  }

  return @file_names;
}

has q{_lane_splitting_cache} => (isa => q{HashRef}, is => q{ro}, lazy_build => 1);

sub _build__lane_splitting_cache {
  my ($self) = @_;
  return {};
}

has q{_lane_phix_splitting_cache} => (isa => q{HashRef}, is => q{ro}, lazy_build => 1);

sub _build__lane_phix_splitting_cache {
  my ($self) = @_;
  return {};
}

=head2 lanes_by_split_nonsplit

given an array of lane numbers, sorts them into groupings

split_nc_lanes
non_split_nc_lanes
split_nc_mp_lanes
non_split_nc_mp_lanes

which is returned as a hashref. Lanes can be provided as an array or arrayref

  my $hLanesBySplitNonsplit = $oMySubClass->lanes_by_split_nonsplit( 1..8 );
  my $hLanesBySplitNonsplit = $oMySubClass->lanes_by_split_nonsplit( [1..8] );

=cut

sub lanes_by_split_nonsplit {
  my ( $self, @lanes ) = @_;

  # deal with arrayref
  if ( ref $lanes[0] && ref $lanes[0] eq q{ARRAY} ) {
    @lanes = @{ $lanes[0] };
  }

  @lanes = sort @lanes; # most likely will be in order, but double check

  my $split_nc_lanes        = [];
  my $non_split_nc_lanes    = [];
  my $split_nc_mp_lanes     = [];
  my $non_split_nc_mp_lanes = [];
  my $recalibrated_lanes     = [];
  my $non_recalibrated_lanes = [];
  my $spiked_phix_lanes      = [];
  my $indexed_lanes     = [];
  my $non_indexed_lanes = [];

  my $return = {
    split_nc_lanes        => $split_nc_lanes,
    non_split_nc_lanes    => $non_split_nc_lanes,
    split_nc_mp_lanes     => $split_nc_mp_lanes,
    non_split_nc_mp_lanes => $non_split_nc_mp_lanes,
    recalibrated_lanes      => $recalibrated_lanes,
    non_recalibrated_lanes  => $non_recalibrated_lanes,
    spiked_phix_lanes       => $spiked_phix_lanes,
    indexed_lanes           => $indexed_lanes,
    non_indexed_lanes       => $non_indexed_lanes,
  };

  foreach my $lane ( @lanes ) {

    if ( $self->_does_lane_need_splitting( $lane ) eq q{true} ) {
      push @{ $split_nc_lanes }, $lane;
      if ( $self->is_indexed() && $self->is_multiplexed_lane( $lane ) ) {
        push @{ $split_nc_mp_lanes }, $lane;
      }
    } else {
      push @{ $non_split_nc_lanes }, $lane;
      if ( $self->is_indexed() && $self->is_multiplexed_lane( $lane ) ) {
        push @{ $non_split_nc_mp_lanes }, $lane;
      }
    }

    if( ! $self->_is_custom_qseq_file_original( $lane ) ){
       push @{$recalibrated_lanes}, $lane;
    }else{
       push @{$non_recalibrated_lanes}, $lane;
    }

    if( ($self->has_all_lanes_as_spiked_phix() &&
        $self->all_lanes_as_spiked_phix() ) ||
        ($self->has_all_lanes_as_non_spiked_phix() &&
        ! $self->all_lanes_as_non_spiked_phix() ) ||
        $self->_is_lane_spiked_phix( $lane ) eq q{true}){

       push @{$spiked_phix_lanes}, $lane;
    }

    if ( $self->is_multiplexed_lane($lane) ) {
      push @{$indexed_lanes}, $lane;
    } else {
      push @{$non_indexed_lanes}, $lane;
    }


  }

  return $return;
}

sub _does_lane_need_splitting {
  my ($self, $lane) = @_;

  # return false if the lane that comes through is not a number
  if ( $lane !~ /\A\d+\z/xms ) {
    return q{false};
  }

  if ($self->_lane_splitting_cache()->{$lane}) {
    return $self->_lane_splitting_cache()->{$lane};
  }

  my $run = $self->run() || $ENV{TEST_RUN_OBJECT};

  my $lane_obj;
  foreach my $l (@{$run->run_lanes()}) {
    if ($l->position() == $lane) {
      $lane_obj = $l;
      last;
    }
  }

  $self->_lane_splitting_cache()->{$lane} = $lane_obj->contains_unconsented_human() ? q{true}
                                          :                                           q{false}
                                          ;

  return $self->_lane_splitting_cache()->{$lane};
}

sub _is_lane_spiked_phix {
   my ($self, $lane) = @_;

   if ( $lane !~ /\A\d+\z/xms ) {
    return q{false};
   }

   if ( $self->has_all_lanes_as_spiked_phix() &&
        $self->all_lanes_as_spiked_phix() ) {
          return q{true};
   }
   if ( $self->has_all_lanes_as_non_spiked_phix() &&
        $self->all_lanes_as_non_spiked_phix() ) {
          return q{false};
   }

   if ( $self->has_all_lanes_as_spiked_phix() &&
        ! $self->all_lanes_as_spiked_phix() ) {
          return q{false};
   }
   if ( $self->has_all_lanes_as_non_spiked_phix() &&
        ! $self->all_lanes_as_non_spiked_phix() ) {
          return q{true};
   }


   if( $self->_lane_phix_splitting_cache()->{$lane} ){
      return $self->_lane_phix_splitting_cache()->{$lane};
   }

   my $run = $self->run() || $ENV{TEST_RUN_OBJECT};

   my $lane_obj;
   foreach my $l (@{$run->run_lanes()}) {
     if ($l->position() == $lane) {
        $lane_obj = $l;
        last;
     }
   }

  $self->_lane_phix_splitting_cache()->{$lane} = $lane_obj->is_spiked_phix() ? q{true}
                                          :                                    q{false}
                                          ;

  return $self->_lane_phix_splitting_cache()->{$lane};
}

=head2 get_fastq_file_by_tag - return the full names with path of fastq by tag files in a hashref

{ both  => {0=> qw(4360_3_1#0.fastq 4360_3_2#0.fastq],
            1=> qw(4360_3_1#1.fastq 4360_3_2#1.fastq],
            ....
           },
  human => {0=> qw(4360_3_1_human#0.fastq 4360_3_2_human#0.fastq],
            1=> qw(4360_3_1_human#1.fastq 4360_3_2_human#1.fastq],
            ....
           },
  nonhuman => {
            }
}

=cut

sub get_fastq_file_by_tag {
   my ($self, $lane) = @_;

   if( !$lane ) {
     $lane = $self->lane();
   }

   if( !$lane ) {
     croak 'no run lane number given';
   }

   my @fastq_filesnames = $self->generate_fastq_and_check_name_parts( { lane => $lane } );
   my $split_fastq = $self->_does_lane_need_splitting( $lane );
   my @tag_index_list = @{ $self->get_tag_index_list( $lane ) };

   my $fastq_files_by_tag = {};

   foreach my $tag_index ( @tag_index_list ) {
     foreach my $part_fastq_name ( @fastq_filesnames ) {

      if ( $part_fastq_name =~ /_t$/mxs ) {
        next;
      }
      my $full_fastq_name = $self->archive_path . qq(/lane$lane) . q[/] . $self->id_run() . q{_} . $part_fastq_name;

      if ( $split_fastq eq q{true} ) {
        my $human_fastq = $full_fastq_name . q{_human} . qq{#$tag_index.fastq};
        my $nonhuman_fastq = $full_fastq_name . q{_nonhuman} . qq{#$tag_index.fastq};
        push @{ $fastq_files_by_tag->{human}->{ $tag_index } }, $human_fastq;
        push @{ $fastq_files_by_tag->{nonhuman}->{ $tag_index } }, $nonhuman_fastq;
      }
      $full_fastq_name .= qq{#$tag_index.fastq};

      push @{ $fastq_files_by_tag->{both}->{ $tag_index } }, $full_fastq_name;
    }
  }
  return $fastq_files_by_tag;
}


sub _is_custom_qseq_file_original {
   my ($self, $lane) = @_;

   my $bustard_path = abs_path( $self->bustard_path() );
   $bustard_path =~ s/\/$//mxs;
   my $qseq_file_name_orig = q{s_}.$lane.q{_1_0001_qseq};
   my $qseq_file_full_orig = File::Spec->catfile ( $bustard_path, $qseq_file_name_orig.q{.txt});

   #if no qseq file in bustard directory, then suppose the custom is original
   if (! -e $qseq_file_full_orig ){
      return 1;
   }

   #check both custom and original qseq file name and path are the same
   my $qseq_file_name_custom      = q{s_}.$lane.q{_1_0001_qseq_custom.txt};
   my $qseq_file_full_custom      = File::Spec->catfile ($self->recalibrated_path(), $qseq_file_name_custom);
   my $qseq_file_full_custom_abs  = abs_path( $qseq_file_full_custom );

   my ($name, $path, $suffix) = fileparse($qseq_file_full_custom_abs, qw(.txt));
   $path =~ s/\/$//mxs;


   if( $name eq $qseq_file_name_orig && $path eq $bustard_path ){
      return 1;
   }

   return 0;
}

#given aligner, species or others in args_ref to get reference in repository
sub _get_reference { ## no critic (Subroutines::ProhibitUnusedPrivateSubroutines)
   my ($self, $args_ref ) = @_;

   if (! $args_ref ) {
      $args_ref = {};
   }

   if ( ! $args_ref->{repository} && $self->has_repository() ) {
     $args_ref->{repository} = $self->repository();
   }

   my $ruser = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_common::sequence::reference::roles::find/])
          ->new_object($args_ref);

   my $reference;
   eval {
      $reference = $ruser->refs->[0];
      1;
   } or do{
      carp $EVAL_ERROR;
   };

   return $reference;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
