#############
# $Id: fastq.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-07
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/archive/file/generation/fastq.pm $

package npg_pipeline::archive::file::generation::fastq;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::archive::file::generation};

sub generate {
  my ($self, $arg_refs) = @_;

  my $id_run = $self->id_run();

  $self->log(qq{Creating Jobs to create fastq files for $id_run});

  my @job_ids;

  my $href = $self->lanes_by_split_nonsplit( $self->positions( $arg_refs ) );

  foreach my $type ( qw{ indexed_lanes non_indexed_lanes } ) {

    next if ! scalar @{ $href->{ $type } };

    $arg_refs->{type_of_lanes} = $type;

    $arg_refs->{array_string} = $self->create_array_string( @{ $href->{ $type } } );

    my @fast_q_file_names = $self->generate_fastq_and_check_name_parts($arg_refs);

    foreach my $file_name (@fast_q_file_names) {
      $arg_refs->{file_name} = $file_name;
      my $job_sub = $self->_generate_bsub_command($arg_refs);
      push @job_ids, $self->submit_bsub_command($job_sub);
    }
  }

  return @job_ids;
}

###############
# responsible for generating the bsub command to be executed for non_indexed runs
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;

  my $file_name = $arg_refs->{file_name};
  my ( $read )  = $arg_refs->{file_name} =~ /_(\w)\z/xms;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp   = $self->timestamp();
  my $script_bin  = $self->internal_script_bin();
  my $id_run      = $self->id_run();

  my $outfile     = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();
  my $job_name    = q{fastq_creation_} . $id_run . q{_} . $timestamp;
  my $replacement = $job_name . q{.%I.%J};
  $outfile        =~ s/substitute_this/$replacement/xms;
  $job_name .= $arg_refs->{array_string};


  my $fast_q_file = $self->archive_path() . q{/} . $self->fastq_file_constructs();
  $fast_q_file    =~ s/substitute_this/$file_name/xms;

  my $md5_name = $fast_q_file . q{.md5};
  my $fqcheck_name = $fast_q_file;
  $fqcheck_name =~ s/fastq/fastqcheck/xms;

  if ( $self->is_indexed() ) {
    my $sanger_read_two = $self->sanger_read_two();
    my $sanger_read_index = $self->sanger_read_index();
    my $illumina_read_two = $self->illumina_read_two();
    my $illumina_read_index = $self->illumina_read_index();
    $file_name =~ s/_$sanger_read_two/_$illumina_read_two/xms;
    $file_name =~ s/_$sanger_read_index/_$illumina_read_index/xms;
  }

  my $qseq_file_string = $self->qseq_file_constructs( $file_name );

  my $job_sub = q{bsub -q } . $self->lsf_queue() . q{ };
  $job_sub .=  ( $self->fs_resource_string( {
    counter_slots_per_job => 4,
  } ) ) . q{ };
  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '$script_bin/} . $self->external_script_names_conf()->{fast_q_creator_script};
  # options required
  $job_sub .= qq{ --md5 $md5_name};
  $job_sub .= qq{ --run $id_run};
  $job_sub .= qq{ --fastqcheck $fqcheck_name};
  if ( $arg_refs->{type_of_lanes} eq q{indexed_lanes} ) {
    $job_sub .= q{ --index};
  }
  if ( $self->is_indexed() ) {
    $job_sub .= qq{ --read $read};
  }
  $job_sub .=  q{ --filter --unique};

  $job_sub .= qq{ $qseq_file_string > $fast_q_file'};

  if ($self->verbose()) { $self->log($job_sub); }
  return $job_sub;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::fastq

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::fastq->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating fastq files.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the fastq files, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
