#############
# $Id: illumina2bam.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: gq1
# Last Maintained By: $Author: ajb $
# Created On: 2011-05-16
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/archive/file/generation/illumina2bam.pm $

package npg_pipeline::archive::file::generation::illumina2bam;

use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;

extends q{npg_pipeline::archive::file::generation};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $DEFAULT_RESOURCES => q{-R 'select[mem>2500] rusage[mem=2500]' -M2500000};
Readonly::Scalar our $ILLUMINA2BAM_CMD  => q{/software/bin/java -Xms512m -Xmx1024m}
                                       . q{ -jar /software/solexa/bin/aligners/illumina2bam/current/illumina2bam.jar}
                                       . q{ CREATE_MD5_FILE=true};
Readonly::Scalar our $BAM_INDEX_DECODE_CMD  => q{/software/bin/java -Xms512m -Xmx1024m}
                                       . q{ -jar /software/solexa/bin/aligners/illumina2bam/current/BamIndexDecoder.jar}
                                       . q{ CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT};

sub generate {
  my ( $self, $arg_refs ) = @_;

  my @positions = $self->positions( $arg_refs );

  $self->log( q{Creating Jobs to run illumina2bam for run } . $self->id_run );

  my $alims = $self->lims->associated_child_lims_ia;
  my @job_ids;
  for my $p (@positions){
    my $bsub_cmd = $self->_generate_bsub_commands( $arg_refs, $alims->{$p});
    push @job_ids, $self->submit_bsub_command( $bsub_cmd );
  }

  return @job_ids;
}

sub _generate_bsub_commands {
  my ( $self, $arg_refs, $lane_lims ) = @_;

  my $position = $lane_lims->position;
  my $required_job_completion = $arg_refs->{required_job_completion};

  my $id_run         = $self->id_run();
  my $intensity_path = $self->intensity_path();
  my $basecall_path  = $self->basecall_path();

  my $full_bam_name  = $basecall_path . q{/}. $id_run . q{_} .$position. q{.bam};

  my $job_name = q{illumina2bam_} . $id_run . q{_} . $position. q{_} . $self->timestamp();

  my $log_folder = $self->make_log_dir( $basecall_path );
  my $outfile = $log_folder . q{/} . $self->bsub_output_file();
  my $out_subs_name = $job_name . q{.%J};
  $outfile =~ s/substitute_this/$out_subs_name/xms;

  $job_name = q{'} . $job_name . q{'};

  my $job_command = $ILLUMINA2BAM_CMD
                  . q{ I=} . $intensity_path
                  . q{ L=} . $position;

  my $st_names = $self->_get_library_sample_study_names($lane_lims);

  if($st_names->{library}){
      $job_command .= q{ LIBRARY_NAME="} . $st_names->{library} . q{"};
  }
  if($st_names->{sample}){
      $job_command .= q{ SAMPLE_ALIAS="} . $st_names->{sample} . q{"};
  }
  if($st_names->{study}){
      my $study = $st_names->{study};
      $study =~ s/"/\\"/gmxs;
      $study =~ s/'/'"'"'/gmxs;
      $job_command .= q{ STUDY_NAME="} . $study . q{"};
  }

  if( ! $self->is_multiplexed_lane($position) ){
      $job_command .= q{ O=} .  $full_bam_name;
  }else{
      $job_command .= q{ O=} . q{/dev/stdout}
                   . q{ COMPRESSION_LEVEL=0}
                   . q{ | }
                   . $BAM_INDEX_DECODE_CMD
                   . q{ I=/dev/stdin }
                   . q{ OUTPUT=}.$full_bam_name
                   . q{ BARCODE_FILE=}.$self->runfolder_path() . qq{/lane_$position.taglist}
                   . q{ METRICS_FILE=} . $full_bam_name . q{.metrics};
  }

  my $resources = ( $self->fs_resource_string( {
      counter_slots_per_job => 1,
      resource_string => $DEFAULT_RESOURCES,
  } ) );

  my $job_sub = q{bsub -q } . $self->lsf_queue() . qq{ $resources $required_job_completion -J $job_name -o $outfile '$job_command'};

  if ( $self->verbose() ) { $self->log($job_sub); }

  return $job_sub;
}

sub _get_library_sample_study_names {

   my ($self, $lane_lims) = @_;

   my $names = $self->get_study_library_sample_names($lane_lims);

   my ($study_names, $library_names, $sample_names);

   if($names->{study}){
      $study_names = join q{,}, @{$names->{study}};
   }
   if($names->{library}){
      $library_names = join q{,}, @{$names->{library}};
   }
   if($names->{sample}){
      $sample_names = join q{,}, @{$names->{sample}};
   }

   my $joined_names = {study=>$study_names, library=>$library_names, sample=>$sample_names};

   return $joined_names;
}

no Moose;

__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::illumina2bam

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating bam files from bcl files.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the fastq files, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2011 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
