#############
# $Id: base.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-11-03
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/roles/business/base.pm $

package npg_pipeline::roles::business::base;
use Moose::Role;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

use File::Slurp;
use lib qw{t};
use Carp;
use Test::More;

use npg::api::util;
use npg::api::run;
use st::api::batch;
use st::api::lims;

Readonly::Scalar our $TILE_LIST     => q[]; # wildcards for tiles to process (NULL indicates all)

=head1 NAME

npg_pipeline::roles::business::base

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::business::base};

=head1 DESCRIPTION

This role provides some base attributes and methods which are business logic related, but likely to be applied over the
whole of the pipeline. For information based items, where there is likely to need to be fixed information, use
npg_pipeline::roles::business::internal_info

=head1 SUBROUTINES/METHODS

=head2 run

npg::api::run object
An id_run method is required for this.

  my $oRun = $class->run();

=cut

has q{run} => (isa => q{Any},  is => q{ro}, lazy_build => 1);

sub _build_run {
  my ($self) = @_;

  if (!$ENV{TEST_USERAGENT} && $ENV{TEST_DIR}) {
    return;
  }

  return npg::api::run->new({
            util   => $self->_apiutil(),
            id_run => $self->id_run(),
          });
}

=head2 instrument_model

The instrument model as obtained from the run object

  my $sInstModel = $oClass->inst_model();

Can be set on the command line

=cut

has q{instrument_model} => (
  isa => q{Str},
  is  => q{ro},
  lazy_build => 1,
  documentation => q{The instrument model type, as it would be described in tracking},
);

sub _build_instrument_model {
  my ( $self ) = @_;
  return $self->run->instrument->model();
}

=head2 batch

st:api::batch object - an id_run method is required, as run will be built to obtain the batch_id

  my $oBatch = $class->batch();

=cut

has q{batch} => (isa => q{Any},  is => q{ro}, lazy_build => 1);

sub _build_batch {
  my ($self) = @_;
  return st::api::batch->new({
    id => $self->run->batch_id(),
    useragent   => $self->run->util->useragent(),
  });
}

=head2 lims

Lims st::api::lims batch-level object

=cut

has q{lims} => (isa => q{st::api::lims},  is => q{ro}, lazy_build => 1);
sub _build_lims {
  my ($self) = @_;
  return st::api::lims->new(id_run => $self->id_run);
}


=head2 positions

return an array of lane positions for this submission. If $arg_refs->{lane} or $self->lane() is provided, will return the contents of this as an array

  my @Positions = $oMySubClass->positions(); # will return the lanes on the run, or if unable (1,2,3,4,5,6,7,8)

  my @Positions = $oMySubClass->positions({
    lane => 5,
  }); # will return an array (5)

  my @Positions = $oMySubClass->positions({
    lane => [1,2,5,8],
  }); # will return an array (1,2,5,8)

=cut

# needs altering to take account of when lanes have been specified on command line

sub positions {
  my ($self, $arg_refs) = @_;

  $arg_refs ||= {};
  my $run = $self->run();

  if (!$arg_refs->{lane} && $self->has_lane()) {
    $arg_refs->{lane} = $self->lane();
  }

  my @positions = $arg_refs->{lane}   ? ($arg_refs->{lane})
                : !$self->no_lanes()  ? $self->all_lanes()
                : $self->lims         ? map {$_->position()} $self->lims->associated_child_lims
                : $run                ? map {$_->position()} @{$run->run_lanes()}
                :                       qw{1 2 3 4 5 6 7 8}
                ;

  @positions = sort @positions;
  return @positions;
}

=head2 processor_fork_number

integer containing the number of processors and forked jobs to be used in an lsf command, where it may need to be variable.

  my $iProcessorForkNumber = $class->processor_fork_number();

=cut

has q{processor_fork_number} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the number of processors and jobs to be forked},);
sub _build_processor_fork_number {
  my ($self) = @_;
  return $self->general_values_conf()->{default_nproc};
}

=head2 fuse_archive

returns the fuse archive location - default /fuse/mpsafs/runs

=cut

sub fuse_archive {
  my ($self) = @_;
  my $test_loc = q{};
  if ($ENV{TEST_DIR}) {
    $test_loc = q{t};
  }
  return $test_loc . $self->filesystem_locations_conf()->{fuse};
}

=head2 fuse_md5_archive

returns the fuse md5 archive location, this needs an id_run passed to it, and uses the location given for fuse_archive

  my $sFuseMd5Archive = $class->fuse_md5_archive($iIdRun);

=cut

sub fuse_md5_archive {
  my ($self, $id_run) = @_;
  if (!$id_run) {
    eval {
      $id_run = $self->id_run();
    } or do {
      croak q{no id_run can be found};
    };
  }
  return $self->fuse_archive . qq{/$id_run/} . $self->_md5();
}

=head2 tile_list

string of wildcards for tiles to be processed (see Illumina Sequencing Analysis Software User Guide)

  my $iTileList = $class->tile_list();

=cut

has q{tile_list} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                      documentation => q{string of wildcards for tiles to be processed (see user guide) - DEFAULT: NULL},);
sub _build_tile_list {
  my ($self) = @_;
  return $TILE_LIST;
}

=head2 webservice_cache_dir

if you already have a cache directory created, you can give this path to the pipeline via this method.

=cut

has q(webservice_cache_dir) => (
  isa => q{Str},
  is => q{ro},
  predicate => q{has_webservice_cache_dir},
  documentation => q{provide a cache directory path to be used},
);

=head2 repository

for passing through a reference repository root directory, should you need to

=cut

has q{repository} => (
  isa       => q{Str},
  is        => q{ro},
  required  => 0,
  predicate => q{has_repository},
);

=head2 ref_repository

for passing through a reference repository, should you need to

=cut

has q{ref_repository} => (
  isa       => q{Str},
  is        => q{ro},
  required  => 0,
  lazy_build => 1,
);

sub _build_ref_repository {
  my ( $self ) = @_;

  return $self->repository() . q{/references};
}

=head2 analyse_rta_log_file

store and hold of analyse_rta_log_file

=head2 pb_cal_log_file

store and hold of pb_cal_log_file

=head2 post_qseq_rta_log_file

store and hold of post_qseq_log_file

=cut

has [ qw{
    analyse_rta_log_file
    pb_cal_log_file
    post_qseq_log_file
  } ] => (
  isa => q{Str},
  is  => q{ro},
  lazy_build => 1,
  builder => q{_build_any_log_file_name},
  documentation => q{only supply this if you know why you should},
);

=head2 instrument_type

The instrument type obtained from NPG tracking

=cut

has q{instrument_type} => (
  isa => q{Str},
  is  => q{ro},
  lazy_build => 1,
  documentation => q{Instrument type that the run was on as obtained from NPG tracking},
);

sub _build_instrument_type {
  my ( $self ) = @_;
  return $self->run->instrument->model();
}

#########
# private

has q{_md5} => (isa => q{Str}, is => q{ro}, lazy_build => 1, writer => q{_test_md5});

sub _build__md5 {
  my ($self) = @_;
  return q{.md5};
}

has q{_apiutil} => (isa => q{npg::api::util}, is => q{ro}, lazy_build => 1, init_arg => undef);

sub _build__apiutil {
  my ($self) = @_;

  my $arg_refs = {};
  if ( $self->_t_useragent() ) {
    $arg_refs->{useragent} = $self->_t_useragent();
  }

  return npg::api::util->new( $arg_refs );

}

#has q{_t_util} => (isa => q{t::util}, is => q{ro}, lazy_build => 1, init_arg => undef);

sub _build__t_util {
  my ($self) = @_;
  require t::util;
  return t::util->new();
}

sub _t_useragent {
  my ( $self ) = @_;
  if ( ! $ENV{TEST_USERAGENT} ) {
    return;
  }
  return $self->_t_useragent_store();
}

has q{_t_useragent_store} => (isa => q{t::useragent}, is => q{ro}, lazy_build => 1, init_arg => undef);

sub _build__t_useragent_store {
  my ($self) = @_;
  my $VAR1;
  require t::useragent;
  eval read_file ( $ENV{TEST_USERAGENT} ); ## no critic (BuiltinFunctions::ProhibitStringyEval ErrorHandling::RequireCheckingReturnValueOfEval)
  return $VAR1;
}

sub _build_any_log_file_name {
  my ( $self ) = @_;
  diag $self->log_file_path() . q{/} . $self->log_file_name();
  return $self->log_file_path() . q{/} . $self->log_file_name();
}

=head2 get_study_library_sample_names

given a position and a tag_index, return a hash with study, library and sample names 

=cut

sub get_study_library_sample_names {
  my ($self, $elims) = @_;

  my $study_names = [];
  my $sample_names = [];

  my @alims = !$elims->is_pool ? ($elims) : $elims->associated_lims;
  foreach my $al (@alims) {
     push @{$sample_names}, $al->sample_publishable_name;
     my $study_name = $al->study_publishable_name();
     my $study_description = $al->study_description;
     if($study_description){
        $study_description =~ s/\n/\ /gmxs;
        $study_description =~ s/\t/\ /gmxs;
        $study_name .= q{: }.$study_description;
     }
     push @{$study_names}, $study_name;
  }

  my $library_aref = $elims->library_id ? [$elims->library_id] : [];
  my $href = {
          study    => $study_names,
          library  => $library_aref,
          sample   => $sample_names,
         };
  return $href;
}

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
